﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;   ///< To use Toradex C Win32 DLLs

namespace vcsharp_characterlcd_demo
{
    public partial class Form1 : Form
    {
        [DllImport("GpioLib.dll", CharSet = CharSet.Auto)]  ///< Importing DLL
        public static extern bool SetPinAltFn(Int32 pinNum, Int32 altFn, bool dirOut); ///< External function declaration
        [DllImport("GpioLib.dll", CharSet = CharSet.Auto)]
        public static extern void InitGPIOLib();
        [DllImport("GpioLib.dll", CharSet = CharSet.Auto)]
        public static extern bool SetPinLevel(Int32 gpioNum, Int32 val);
        [DllImport("GpioLib.dll", CharSet = CharSet.Auto)]
        public static extern void DeInitGPIOLib();
        
        public Form1()
        {
            InitializeComponent();
            InitGPIOLib();
            InitLcdPins();
            InitLcd();
        }
        
        public void button_line1_Click(object sender, EventArgs e)
        {
            string line1String = this.textBoxLine1.Text;
            Int32 strLength = line1String.Length;
            Globals.line1Data = System.Text.Encoding.ASCII.GetBytes(line1String); 
            PutStringOnLcd(0x80, 0, strLength);
            SendByteToLCD(0x00, Globals.command);
        }

        private void button_line2_Click(object sender, EventArgs e)
        {
            string line2String = this.textBoxLine2.Text;
            Int32 strLength = line2String.Length;
            Globals.line2Data = System.Text.Encoding.ASCII.GetBytes(line2String); 
            PutStringOnLcd(0xC0, 0, strLength);
            SendByteToLCD(0x00, Globals.command);
        }

        private void button_line3_Click(object sender, EventArgs e)
        {
            string line3String = this.textBoxLine3.Text;
            Int32 strLength = line3String.Length;
            Globals.line3Data = System.Text.Encoding.ASCII.GetBytes(line3String); 
            PutStringOnLcd(0x94, 0, strLength);
            SendByteToLCD(0x00, Globals.command);
        }

        private void button_line4_Click(object sender, EventArgs e)
        {
            string line4String = this.textBoxLine4.Text;
            Int32 strLength = line4String.Length;
            Globals.line4Data = System.Text.Encoding.ASCII.GetBytes(line4String);
            PutStringOnLcd(0xD4, 0, strLength);
            SendByteToLCD(0x00, Globals.command);
        }

        public static class Globals
        {
            public static Int32 d4Pin = 85;
            public static Int32 d5Pin = 97;
            public static Int32 d6Pin = 101;
            public static Int32 d7Pin = 103;
            public static Int32 enPin = 79;
            public static Int32 rsPin = 45;

            public static Int32 setPin = 1;
            public static Int32 clearPin = 0;

            public static bool command = false;
            public static bool data = true;

            public static byte[] line1Data = new byte[20];
            public static byte[] line2Data = new byte[20];
            public static byte[] line3Data = new byte[20];
            public static byte[] line4Data = new byte[20];
        }
        private void InitLcdPins()
        {
            /// Pin direction configuration (output)
            SetPinAltFn(Globals.d4Pin, -1, true);       ///< D4
            SetPinAltFn(Globals.d5Pin, -1, true);       ///< D5
            SetPinAltFn(Globals.d6Pin, -1, true);       ///< D6
            SetPinAltFn(Globals.d7Pin, -1, true);       ///< D7
            SetPinAltFn(Globals.rsPin, -1, true);       ///< RS
            SetPinAltFn(Globals.enPin, -1, true);       ///< EN
        }

        private void InitLcd()
        {
            byte[] lcdInitCommand = new byte[7] {0x20,      ///< 4-bit interface
                                              0x20,         ///< 4-bit interface
                                              0x20,         ///< 4-bit interface
                                              0x28,         ///< 4-bit, 2 line
                                              0x0C,         ///< Display ON, cursor OFF
                                              0x06,         ///< Display ON, cursor OFF
                                              0x01 };       ///< Clear LCD

            for (Int16 counter = 0; counter < 4; counter++)
            {
                SetPinLevel(Globals.d7Pin, Globals.clearPin);
                SetPinLevel(Globals.d6Pin, Globals.clearPin);
                SetPinLevel(Globals.d5Pin, Globals.setPin);
                if (counter == 3)
                {
                    SetPinLevel(Globals.d4Pin, Globals.clearPin);
                }
                else
                {
                    SetPinLevel(Globals.d4Pin, Globals.setPin);
                }
                SetPinLevel(Globals.enPin, Globals.setPin);
                System.Threading.Thread.Sleep(1);
                SetPinLevel(Globals.enPin, Globals.clearPin);
                System.Threading.Thread.Sleep(1);
            }

            for (byte tmp = 0; tmp < 7; tmp++)
            {
                SetPinLevel(Globals.rsPin, Globals.clearPin);
                SendByteToLCD(lcdInitCommand[tmp], Globals.command);
                System.Threading.Thread.Sleep(80);
            }
        }

        private void SendByteToLCD(byte rawByte, bool type)
        {
            /// COMMAND or DATA selection (RS)
            if (type == true)
                SetPinLevel(Globals.rsPin, Globals.setPin);
            else
                SetPinLevel(Globals.rsPin, Globals.clearPin);

            /// Higher nibble
            if ((rawByte & 0x10) > 0)
                SetPinLevel(Globals.d4Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d4Pin, Globals.clearPin);

            if ((rawByte & 0x20) > 0)
                SetPinLevel(Globals.d5Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d5Pin, Globals.clearPin);

            if ((rawByte & 0x40) > 0)
                SetPinLevel(Globals.d6Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d6Pin, Globals.clearPin);

            if ((rawByte & 0x80) > 0)
                SetPinLevel(Globals.d7Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d7Pin, Globals.clearPin);

            SetPinLevel(Globals.enPin, Globals.setPin);
            System.Threading.Thread.Sleep(1);
            SetPinLevel(Globals.enPin, Globals.clearPin);
            System.Threading.Thread.Sleep(1);

            /// Lower nibble
            if ((rawByte & 0x01) > 0)
                SetPinLevel(Globals.d4Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d4Pin, Globals.clearPin);

            if ((rawByte & 0x02) > 0)
                SetPinLevel(Globals.d5Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d5Pin, Globals.clearPin);

            if ((rawByte & 0x04) > 0)
                SetPinLevel(Globals.d6Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d6Pin, Globals.clearPin);

            if ((rawByte & 0x08) > 0)
                SetPinLevel(Globals.d7Pin, Globals.setPin);
            else
                SetPinLevel(Globals.d7Pin, Globals.clearPin);

            SetPinLevel(Globals.enPin, Globals.setPin);
            System.Threading.Thread.Sleep(1);
            SetPinLevel(Globals.enPin, Globals.clearPin);
            System.Threading.Thread.Sleep(1);
        }

        public void PutStringOnLcd(byte lineNumber, byte stringStartPosition, Int32 strLength)
        {
            SendByteToLCD(((byte)(lineNumber + stringStartPosition)), Globals.command);

            for (Int32 count = 0; count < strLength; count++)
            {
                switch (lineNumber)
                {
                    case 0x80:
                        SendByteToLCD(Globals.line1Data[count], Globals.data);
                        break;
                    case 0xC0:
                        SendByteToLCD(Globals.line2Data[count], Globals.data);
                        break;
                    case 0x94:
                        SendByteToLCD(Globals.line3Data[count], Globals.data);
                        break;
                    case 0xD4:
                        SendByteToLCD(Globals.line4Data[count], Globals.data);
                        break;
                    default:
                        break;
                }
            }

        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            DeInitGPIOLib();            ///< De-initializes GPIO library and release system resources upon closing the program
        }
    }
}